# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 David Planella <david.planella@ubuntu.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import os

TEXTDOMAIN = 'qreator'
LOCALEDIR = '/usr/share/locale/'

import locale
from locale import gettext as _

if os.path.abspath(__file__).startswith('/opt'):
    LOCALEDIR = '/opt/extras.ubuntu.com/qreator/share/locale/'

locale.bindtextdomain(TEXTDOMAIN, LOCALEDIR)
locale.bind_textdomain_codeset(TEXTDOMAIN, 'UTF-8')
locale.textdomain(TEXTDOMAIN)
